@@ -1,62 +1,84 @@
-#' Convert many input types with spatial data to geojson specified as a json string
+#' Convert many input types with spatial data to geojson specified as a json
+#' string
 #'
 #' @export
 #'
-#' @param input Input list, data.frame, spatial class, or sf class. Inputs can also be dplyr \code{tbl_df}
-#' class since it inherits from \code{data.frame}.
-#' @param lat (character) Latitude name. The default is \code{NULL}, and we attempt to guess.
-#' @param lon (character) Longitude name. The default is \code{NULL}, and we attempt to guess.
+#' @param input Input list, data.frame, spatial class, or sf class. Inputs can
+#' also be dplyr \code{tbl_df} class since it inherits from \code{data.frame}.
+#' @param lat (character) Latitude name. The default is \code{NULL}, and we
+#' attempt to guess.
+#' @param lon (character) Longitude name. The default is \code{NULL}, and we
+#' attempt to guess.
 #' @param geometry (character) One of point (Default) or polygon.
-#' @param type  (character)The type of collection. One of FeatureCollection (default) or GeometryCollection.
-#' @param group (character) A grouping variable to perform grouping for polygons - doesn't
-#' apply for points
-#' @param convert_wgs84 Should the input be converted to the \href{https://tools.ietf.org/html/rfc7946}{standard coordinate reference system defined for GeoJSON} (geographic coordinate reference system, using the WGS84 datum, with longitude and latitude units of decimal degrees; EPSG: 4326). Default is \code{FALSE} though this may change in a future package version. This will only work for \code{sf} or \code{Spatial} objects with a CRS already defined. If one is not defined but you know what it is, you may define it in the \code{crs} argument below.
-#' @param crs The CRS of the input if it is not already defined. This can be an epsg code as a four or five digit integer or a valid proj4 string. This argument will be ignored if \code{convert_wgs84} is \code{FALSE} or the object already has a CRS.
-#' @param ... Further args passed on to internal functions. For Spatial* classes, it is passed through to 
-#' \code{\link[rgdal]{writeOGR}}. For sf classes, data.frames, lists, numerics, and 
-#' geo_lists, it is passed through to \code{\link[jsonlite]{toJSON}}.
+#' @param type  (character) The type of collection. One of 'auto' (default
+#' for 'sf' objects),
+#' 'FeatureCollection' (default for everything else), or 'GeometryCollection'.
+#' This is ignored for \code{Spatial} objects as it will always produce a
+#' "FeatureCollection"
+#' @param group (character) A grouping variable to perform grouping for
+#' polygons - doesn't apply for points
+#' @param convert_wgs84 Should the input be converted to the
+#' \href{https://tools.ietf.org/html/rfc7946}{standard coordinate reference
+#' system defined for GeoJSON}  (geographic coordinate reference system, using
+#' the WGS84 datum, with longitude and latitude units of decimal degrees;
+#' EPSG: 4326). Default is \code{FALSE} though this may change in a future
+#' package version. This will only work for \code{sf} or \code{Spatial}
+#' objects with a CRS already defined. If one is not defined but you know
+#' what it is, you may define it in the \code{crs} argument below.
+#' @param crs The CRS of the input if it is not already defined. This can be
+#' an epsg code as a four or five digit integer or a valid proj4 string.
+#' This argument will be ignored if \code{convert_wgs84} is \code{FALSE} or
+#' the object already has a CRS.
+#' @param ... Further args passed on to internal functions. For Spatial*
+#' classes, it is passed through to
+#' \code{\link[rgdal]{writeOGR}}. For sf classes, data.frames, lists, numerics,
+#' and geo_lists, it is passed through to \code{\link[jsonlite]{toJSON}}.
 #'
 #' @return An object of class \code{geo_json} (and \code{json})
 #'
-#' @details This function creates a geojson structure as a json character string; it does not
-#' write a file using \code{rgdal} - see \code{\link{geojson_write}} for that.
+#' @details This function creates a geojson structure as a json character
+#' string; it does not write a file using \code{rgdal} - see
+#' \code{\link{geojson_write}} for that.
 #'
-#' Note that all sp class objects will output as \code{FeatureCollection} objects, while other
-#' classes (numeric, list, data.frame) can be output as \code{FeatureCollection} or
-#' \code{GeometryCollection} objects. We're working on allowing \code{GeometryCollection}
-#' option for sp class objects.
+#' Note that all sp class objects will output as \code{FeatureCollection}
+#' objects, while other classes (numeric, list, data.frame) can be output as
+#' \code{FeatureCollection} or \code{GeometryCollection} objects. We're working
+#' on allowing \code{GeometryCollection} option for sp class objects.
+#'
+#' Also note that with sp classes we do make a round-trip, using
+#' \code{\link[rgdal]{writeOGR}} to write GeoJSON to disk, then read it back
+#' in. This is fast and we don't have to think about it too much, but this
+#' disk round-trip is not ideal.
 #'
-#' Also note that with sp classes we do make a round-trip, using \code{\link[rgdal]{writeOGR}}
-#' to write GeoJSON to disk, then read it back in. This is fast and we don't have to think
-#' about it too much, but this disk round-trip is not ideal.
-#' 
 #' For sf classes (sf, sfc, sfg), the following conversions are made:
-#' 
+#'
 #' \itemize{
-#'  \item sfg: the appropriate geometry \code{Point, LineString, Polygon, MultiPoint, 
-#'  MultiLineString, MultiPolygon, GeometryCollection}
-#'  \item sfc: \code{GeometryCollection}, unless the sfc is length 1, then the geometry as above
+#'  \item sfg: the appropriate geometry \code{Point, LineString, Polygon,
+#'  MultiPoint, MultiLineString, MultiPolygon, GeometryCollection}
+#'  \item sfc: \code{GeometryCollection}, unless the sfc is length 1, then
+#'  the geometry as above
 #'  \item sf: \code{FeatureCollection}
 #' }
 #'
 #' @examples \dontrun{
 #' # From a numeric vector of length 2, making a point type
-#' geojson_json(c(-99.74,32.45), pretty=TRUE)
-#' geojson_json(c(-99.74,32.45), type = "GeometryCollection", pretty=TRUE)
+#' geojson_json(c(-99.74,32.45))
+#' geojson_json(c(-99.74,32.45), type = "GeometryCollection")
 #'
 #' ## polygon type
-#' ### this requires numeric class input, so inputting a list will dispatch on the list method
+#' ### this requires numeric class input, so inputting a list will dispatch
+#' ### on the list method
 #' poly <- c(c(-114.345703125,39.436192999314095),
 #'           c(-114.345703125,43.45291889355468),
 #'           c(-106.61132812499999,43.45291889355468),
 #'           c(-106.61132812499999,39.436192999314095),
 #'           c(-114.345703125,39.436192999314095))
-#' geojson_json(poly, geometry = "polygon", pretty=TRUE)
+#' geojson_json(poly, geometry = "polygon")
 #'
 #' # Lists
 #' ## From a list of numeric vectors to a polygon
 #' vecs <- list(c(100.0,0.0), c(101.0,0.0), c(101.0,1.0), c(100.0,1.0), c(100.0,0.0))
-#' geojson_json(vecs, geometry="polygon", pretty=TRUE)
+#' geojson_json(vecs, geometry="polygon")
 #'
 #' ## from a named list
 #' mylist <- list(list(latitude=30, longitude=120, marker="red"),
@@ -64,9 +86,9 @@
 #' geojson_json(mylist, lat='latitude', lon='longitude')
 #'
 #' # From a data.frame to points
-#' geojson_json(us_cities[1:2,], lat='lat', lon='long', pretty=TRUE)
+#' geojson_json(us_cities[1:2,], lat='lat', lon='long')
 #' geojson_json(us_cities[1:2,], lat='lat', lon='long',
-#'    type="GeometryCollection", pretty=TRUE)
+#'    type="GeometryCollection")
 #'
 #' # from data.frame to polygons
 #' head(states)
@@ -87,7 +109,7 @@
 #'    c(30,40,35,30)))), "2")
 #' sp_poly <- SpatialPolygons(list(poly1, poly2), 1:2)
 #' geojson_json(sp_poly)
-#' geojson_json(sp_poly, pretty=TRUE)
+#' geojson_json(sp_poly)
 #'
 #' ## Another SpatialPolygons
 #' library("sp")
@@ -136,7 +158,7 @@
 #'                  Z = c("Road", "River"), row.names = c("a", "b"))
 #' sldf <- SpatialLinesDataFrame(sl12, dat)
 #' geojson_json(sldf)
-#' geojson_json(sldf, pretty=TRUE)
+#' geojson_json(sldf)
 #'
 #' ## From SpatialGrid
 #' x <- GridTopology(c(0,0), c(1,1), c(5,5))
@@ -183,7 +205,7 @@
 #' poly <- SpatialPolygons(list(poly1, poly2), 1:2)
 #' dat <- SpatialCollections(pts, polygons = poly)
 #' geojson_json(dat)
-#' 
+#'
 #' # From sf classes:
 #' if (require(sf)) {
 #' ## sfg (a single simple features geometry)
@@ -191,13 +213,13 @@
 #'   poly <- rbind(c(1,1), c(1,2), c(2,2), c(1,1))
 #'   poly_sfg <-st_polygon(list(p1))
 #'   geojson_json(poly_sfg)
-#'   
+#'
 #' ## sfc (a collection of geometries)
 #'   p1 <- rbind(c(0,0), c(1,0), c(3,2), c(2,4), c(1,4), c(0,0))
 #'   p2 <- rbind(c(5,5), c(5,6), c(4,5), c(5,5))
 #'   poly_sfc <- st_sfc(st_polygon(list(p1)), st_polygon(list(p2)))
 #'   geojson_json(poly_sfc)
-#'   
+#'
 #' ## sf (collection of geometries with attributes)
 #'   p1 <- rbind(c(0,0), c(1,0), c(3,2), c(2,4), c(1,4), c(0,0))
 #'   p2 <- rbind(c(5,5), c(5,6), c(4,5), c(5,5))
@@ -205,7 +227,7 @@
 #'   poly_sf <- st_sf(foo = c("a", "b"), bar = 1:2, poly_sfc)
 #'   geojson_json(poly_sf)
 #' }
-#' 
+#'
 #' ## Pretty print a json string
 #' geojson_json(c(-99.74,32.45))
 #' geojson_json(c(-99.74,32.45)) %>% pretty
@@ -221,15 +243,15 @@ geojson_json <- function(input, lat = NULL, lon = NULL, group = NULL,
 geojson_json.SpatialPolygons <- function(input, lat = NULL, lon = NULL, group = NULL,
                                          geometry = "point",  type='FeatureCollection',
                                          convert_wgs84 = FALSE, crs = NULL, ...) {
-  class_json(geojson_rw(input, target = "char", convert_wgs84 = convert_wgs84, crs = crs), ...)
+  geoclass(geojson_rw(input, target = "char", convert_wgs84 = convert_wgs84, crs = crs), type = 'FeatureCollection')
 }
 
 #' @export
-geojson_json.SpatialPolygonsDataFrame <- function(input, lat = NULL, lon = NULL, 
-                                                  group = NULL, geometry = "point",  
+geojson_json.SpatialPolygonsDataFrame <- function(input, lat = NULL, lon = NULL,
+                                                  group = NULL, geometry = "point",
                                                   type='FeatureCollection',
                                                   convert_wgs84 = FALSE, crs = NULL, ...) {
-  class_json(geojson_rw(input, target = "char", convert_wgs84 = convert_wgs84, crs = crs), ...)
+  geoclass(geojson_rw(input, target = "char", convert_wgs84 = convert_wgs84, crs = crs), type = 'FeatureCollection')
 }
 
 #' @export
@@ -237,83 +259,83 @@ geojson_json.SpatialPoints <- function(input, lat = NULL, lon = NULL, group = NU
                                        geometry = "point",  type='FeatureCollection',
                                        convert_wgs84 = FALSE, crs = NULL, ...) {
   dat <- SpatialPointsDataFrame(input, data.frame(dat = 1:NROW(input@coords)))
-  class_json(geojson_rw(dat, target = "char", convert_wgs84 = convert_wgs84, crs = crs), ...)
+  geoclass(geojson_rw(dat, target = "char", convert_wgs84 = convert_wgs84, crs = crs), type = 'FeatureCollection')
 }
 
 #' @export
-geojson_json.SpatialPointsDataFrame <- function(input, lat = NULL, lon = NULL, 
-                                                group = NULL, geometry = "point",  
+geojson_json.SpatialPointsDataFrame <- function(input, lat = NULL, lon = NULL,
+                                                group = NULL, geometry = "point",
                                                 type='FeatureCollection',
                                                 convert_wgs84 = FALSE, crs = NULL, ...) {
-  class_json(geojson_rw(input, target = "char", convert_wgs84 = convert_wgs84, crs = crs), ...)
+  geoclass(geojson_rw(input, target = "char", convert_wgs84 = convert_wgs84, crs = crs), type = 'FeatureCollection')
 }
 
 #' @export
 geojson_json.SpatialLines <- function(input, lat = NULL, lon = NULL, group = NULL,
                                       geometry = "point",  type='FeatureCollection',
                                       convert_wgs84 = FALSE, crs = NULL, ...) {
-  class_json(geojson_rw(input, target = "char", convert_wgs84 = convert_wgs84, crs = crs), ...)
+  geoclass(geojson_rw(input, target = "char", convert_wgs84 = convert_wgs84, crs = crs), type = 'FeatureCollection')
 }
 
 #' @export
-geojson_json.SpatialLinesDataFrame <- function(input, lat = NULL, lon = NULL, 
-                                               group = NULL, geometry = "point",  
+geojson_json.SpatialLinesDataFrame <- function(input, lat = NULL, lon = NULL,
+                                               group = NULL, geometry = "point",
                                                type='FeatureCollection',
                                                convert_wgs84 = FALSE, crs = NULL, ...) {
-  class_json(geojson_rw(input, target = "char", convert_wgs84 = convert_wgs84, crs = crs), ...)
+  geoclass(geojson_rw(input, target = "char", convert_wgs84 = convert_wgs84, crs = crs), type = 'FeatureCollection')
 }
 
 #' @export
 geojson_json.SpatialGrid <- function(input, lat = NULL, lon = NULL, group = NULL,
                                      geometry = "point",  type='FeatureCollection',
                                      convert_wgs84 = FALSE, crs = NULL, ...) {
-  class_json(geojson_rw(input, target = "char", convert_wgs84 = convert_wgs84, crs = crs), ...)
+  geoclass(geojson_rw(input, target = "char", convert_wgs84 = convert_wgs84, crs = crs), type = 'FeatureCollection')
 }
 
 #' @export
-geojson_json.SpatialGridDataFrame <- function(input, lat = NULL, lon = NULL, 
-                                              group = NULL, geometry = "point",  
+geojson_json.SpatialGridDataFrame <- function(input, lat = NULL, lon = NULL,
+                                              group = NULL, geometry = "point",
                                               type='FeatureCollection',
                                               convert_wgs84 = FALSE, crs = NULL, ...) {
-  class_json(geojson_rw(input, target = "char", convert_wgs84 = convert_wgs84, crs = crs), ...)
+  geoclass(geojson_rw(input, target = "char", convert_wgs84 = convert_wgs84, crs = crs), type = 'FeatureCollection')
 }
 
 #' @export
 geojson_json.SpatialPixels <- function(input, lat = NULL, lon = NULL, group = NULL,
                                        geometry = "point",  type='FeatureCollection',
                                        convert_wgs84 = FALSE, crs = NULL, ...) {
-  class_json(geojson_rw(input, target = "char", convert_wgs84 = convert_wgs84, crs = crs), ...)
+  geoclass(geojson_rw(input, target = "char", convert_wgs84 = convert_wgs84, crs = crs), type = 'FeatureCollection')
 }
 
 #' @export
-geojson_json.SpatialPixelsDataFrame <- function(input, lat = NULL, lon = NULL, 
-                                                group = NULL, geometry = "point",  
+geojson_json.SpatialPixelsDataFrame <- function(input, lat = NULL, lon = NULL,
+                                                group = NULL, geometry = "point",
                                                 type='FeatureCollection',
                                                 convert_wgs84 = FALSE, crs = NULL, ...) {
-  class_json(geojson_rw(input, target = "char", convert_wgs84 = convert_wgs84, crs = crs), ...)
+  geoclass(geojson_rw(input, target = "char", convert_wgs84 = convert_wgs84, crs = crs), type = 'FeatureCollection')
 }
 
 # sf classes ---------------------------------
 
 #' @export
 geojson_json.sf <- function(input, lat = NULL, lon = NULL, group = NULL,
-                            geometry = "point",  type='FeatureCollection',
+                            geometry = "point",  type='auto',
                             convert_wgs84 = FALSE, crs = NULL, ...) {
-  as.json(geojson_list(input, convert_wgs84 = convert_wgs84, crs = crs), ...)
+  geoclass(as.json(geojson_list(input, convert_wgs84 = convert_wgs84, crs = crs), ...), type)
 }
 
 #' @export
 geojson_json.sfc <- function(input, lat = NULL, lon = NULL, group = NULL,
-                             geometry = "point",  type='FeatureCollection',
+                             geometry = "point",  type='auto',
                              convert_wgs84 = FALSE, crs = NULL, ...) {
-  as.json(geojson_list(input, convert_wgs84 = convert_wgs84, crs = crs), ...)
+  geoclass(as.json(geojson_list(input, convert_wgs84 = convert_wgs84, crs = crs), ...), type)
 }
 
 #' @export
 geojson_json.sfg <- function(input, lat = NULL, lon = NULL, group = NULL,
-                             geometry = "point",  type='FeatureCollection',
+                             geometry = "point",  type='auto',
                              convert_wgs84 = FALSE, crs = NULL, ...) {
-  as.json(geojson_list(input), ...)
+  geoclass(as.json(geojson_list(input), ...), type)
 }
 
 # spatial classes from rgeos --------------------------
@@ -321,43 +343,50 @@ geojson_json.sfg <- function(input, lat = NULL, lon = NULL, group = NULL,
 geojson_json.SpatialRings <- function(input, lat = NULL, lon = NULL, group = NULL,
                                       geometry = "point",  type='FeatureCollection',
                                       convert_wgs84 = FALSE, crs = NULL, ...) {
-  class_json(geojson_rw(input, target = "char", convert_wgs84 = convert_wgs84, crs = crs), ...)
+  geoclass(geojson_rw(input, target = "char", convert_wgs84 = convert_wgs84, crs = crs), type = 'FeatureCollection')
 }
 
 #' @export
-geojson_json.SpatialRingsDataFrame <- function(input, lat = NULL, lon = NULL, 
-                                               group = NULL, geometry = "point",  
+geojson_json.SpatialRingsDataFrame <- function(input, lat = NULL, lon = NULL,
+                                               group = NULL, geometry = "point",
                                                type='FeatureCollection',
                                                convert_wgs84 = FALSE, crs = NULL, ...) {
-  class_json(geojson_rw(input, target = "char", convert_wgs84 = convert_wgs84, crs = crs), ...)
+  geoclass(geojson_rw(input, target = "char", convert_wgs84 = convert_wgs84, crs = crs), type = 'FeatureCollection')
 }
 
 #' @export
-geojson_json.SpatialCollections <- function(input, lat = NULL, lon = NULL, 
-                                            group = NULL, geometry = "point",  
+geojson_json.SpatialCollections <- function(input, lat = NULL, lon = NULL,
+                                            group = NULL, geometry = "point",
                                             type='FeatureCollection',
                                             convert_wgs84 = FALSE, crs = NULL, ...) {
-  lapply(geojson_rw(input, target = "char", convert_wgs84 = convert_wgs84, crs = crs, ...), class_json)
+  lapply(
+    geojson_rw(input, target = "char", convert_wgs84 = convert_wgs84, crs = crs, ...),
+    geoclass,
+    type = 'FeatureCollection'
+  )
 }
 
 # regular R classes --------------------------
 #' @export
 geojson_json.numeric <- function(input, lat = NULL, lon = NULL, group = NULL,
                                  geometry = "point", type='FeatureCollection', ...) {
-  to_json(num_to_geo_list(input, geometry, type), ...)
+  check_type(type)
+  geoclass(to_json(num_to_geo_list(input, geometry, type), ...), type)
 }
 
 #' @export
 geojson_json.data.frame <- function(input, lat = NULL, lon = NULL, group = NULL,
                                     geometry = "point", type='FeatureCollection', ...) {
+  check_type(type)
   tmp <- guess_latlon(names(input), lat, lon)
   res <- df_to_geo_list(input, tmp$lat, tmp$lon, geometry, type, group)
-  to_json(res, ...)
+  geoclass(to_json(res, ...), type)
 }
 
 #' @export
 geojson_json.list <- function(input, lat = NULL, lon = NULL, group = NULL,
-                              geometry = "point",  type='FeatureCollection', ...){
+                              geometry = "point", type='FeatureCollection', ...){
+  check_type(type)
   if (geometry == "polygon") lint_polygon_list(input)
   tmp <- if (!is.named(input)) {
     list(lon = NULL, lat = NULL)
@@ -365,12 +394,12 @@ geojson_json.list <- function(input, lat = NULL, lon = NULL, group = NULL,
     guess_latlon(names(input[[1]]), lat, lon)
   }
   res <- list_to_geo_list(input, tmp$lat, tmp$lon, geometry, type, unnamed = !is.named(input), group)
-  to_json(res, ...)
+  geoclass(to_json(res, ...), type)
 }
 
 #' @export
 geojson_json.geo_list <- function(input, lat = NULL, lon = NULL, group = NULL,
                                   geometry = "point", type = "FeatureCollection", ...) {
-  
-  to_json(unclass(input), ...)
+
+  geoclass(to_json(input, ...), type)
 }
